//: C02:MemCheck.cpp {O}
#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <cstddef>
using namespace std;
#undef new

// Globln pznaky nastavovan makry ze souboru MemCheck.h

bool traceFlag = true;
bool activeFlag = false;

namespace {

// Typ pedstavujc poloku mapy pamti
struct Info {
  void* ptr;
  const char* file;
  long line;
};

// data mapy pamti
const size_t MAXPTRS = 10000u;
Info memMap[MAXPTRS];
size_t nptrs = 0;

// hled adresu v map
int findPtr(void* p) {
  for(size_t i = 0; i < nptrs; ++i)
    if(memMap[i].ptr == p)
      return i;
  return -1;
}

void delPtr(void* p) {
  int pos = findPtr(p);
  assert(pos >= 0);
  // odstranit ukazatel z mapy
  for(size_t i = pos; i < nptrs-1; ++i)
    memMap[i] = memMap[i+1];
  --nptrs;
}

// nhrakov typ pro statick destruktor
struct Sentinel {
  ~Sentinel() {
    if(nptrs > 0) {
      printf("Unikl pam na:\n");
      for(size_t i = 0; i < nptrs; ++i)
        printf("\t%p (Soubor: %s, dek %ld)\n",
          memMap[i].ptr, memMap[i].file, memMap[i].line);
    }
    else
      printf("dn uivatelsk pam neunik!\n");
  }
};

// statick nhrakov objekt
Sentinel s;

} // konec anonymnho prostoru jmen

// petit skalrn opertor new
void*
operator new(size_t siz, const char* file, long line) {
  void* p = malloc(siz);
  if(activeFlag) {
    if(nptrs == MAXPTRS) {
      printf("Pli mal mapa pamti (zvtte MAXPTRS)\n");
      exit(1);
    }
    memMap[nptrs].ptr = p;
    memMap[nptrs].file = file;
    memMap[nptrs].line = line;
    ++nptrs;
  }
  if(traceFlag) {
    printf("Alokovno %u byt na adrese %p ", siz, p);
    printf("(soubor: %s, dek: %ld)\n", file, line);
  }
  return p;
}

// petit opertor new pro pole
void*
operator new[](size_t siz, const char* file, long line) {
  return operator new(siz, file, line);
}

// Petit skalrn opertor delete
void operator delete(void* p) {
  if(findPtr(p) >= 0) {
    free(p);
    assert(nptrs > 0);
    delPtr(p);
    if(traceFlag)
      printf("Uvolnna pam na adrese %p\n", p);
  }
  else if(p && activeFlag)
    printf("Pokus o smazn neznmho ukazatele: %p\n", p);
}

// Petit opertor delete pro pole
void operator delete[](void* p) {
  operator delete(p);
} ///:~
