//: C07:BankTeller.cpp {RunByHand}
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// Pouit queue a simulovanho vce vlknovho prosted 
// k modelovn systmu bankovnch pokladnk.
#include <cstdlib>
#include <ctime>
#include <iostream>
#include <iterator>
#include <list>
#include <queue>
using namespace std;

class Zakaznik {
  int dobaObsluhy;
public:
  Zakaznik() : dobaObsluhy(0) {}
  Zakaznik(int tm) : dobaObsluhy(tm) {}
  int zjistiCas() { return dobaObsluhy; }
  void nastavCas(int novyCas) { dobaObsluhy = novyCas; }
  friend ostream&
  operator<<(ostream& os, const Zakaznik& c) {
    return os << '[' << c.dobaObsluhy << ']';
  }
};

class Pokladnik {
  queue<Zakaznik>& zakaznici;
  Zakaznik aktualni;
  enum { PODIL = 5 };
  int zcas; // as zbvajc z podlu
  bool zamestnan; // Obsluhuje pokladnk zkaznka?
public:
  Pokladnik(queue<Zakaznik>& cq)
  : zakaznici(cq), zcas(0), zamestnan(false) {}
  Pokladnik& operator=(const Pokladnik& rv) {
    zakaznici = rv.zakaznici;
    aktualni = rv.aktualni;
    zcas = rv.zcas;
    zamestnan = rv.zamestnan;
    return *this;
  }
  bool jeZamestnan() { return zamestnan; }
  void run(bool rekurze = false) {
    if(!rekurze)
      zcas = PODIL;
    int dobaObsluhovani = aktualni.zjistiCas();
    if(dobaObsluhovani > zcas) {
      dobaObsluhovani -= zcas;
      aktualni.nastavCas(dobaObsluhovani);
      zamestnan = true; // Stle pracuje na aktualnim zkaznkovi
      return;
    }
    if(dobaObsluhovani < zcas) {
      zcas -= dobaObsluhovani;
      if(!zakaznici.empty()) {
        aktualni = zakaznici.front();
        zakaznici.pop(); // Odstranit zakaznika
        zamestnan = true;
        run(true); // Rekurze
      }
      return;
    }
    if(dobaObsluhovani == zcas) {
      //Aktulnk zkaznk vyzen, vyprzdnit:
      aktualni = Zakaznik(0);
      zamestnan = false;
      return; // V tomto podlu u nen dn as
    }
  }
};

//Ddinost zskat pstup k chrnn implementaci:
class FrontaZakazniku : public queue<Zakaznik> {
public:
  friend ostream&
  operator<<(ostream& os, const FrontaZakazniku& cd) {
    copy(cd.c.begin(), cd.c.end(),
      ostream_iterator<Zakaznik>(os, ""));
    return os;
  }
};

int main() {
  FrontaZakazniku zakaznici;
  list<Pokladnik> pokladnici;
  typedef list<Pokladnik>::iterator PoklIt;
  pokladnici.push_back(Pokladnik(zakaznici));
  srand(time(0)); // inicializovat genertor nhodnch sel
  clock_t tiky = clock();
  // Spustit simulaci nejmn na 5 sekund:
  while(clock() < tiky + 5 * CLOCKS_PER_SEC) {
  	//Vloit do fronty nhodn poet zkaznk, 
		//s nhodn dlouhou dobou obsluhy:
    for(int i = 0; i < rand() % 5; i++)
      zakaznici.push(Zakaznik(rand() % 15 + 1));
    cout << '{' << pokladnici.size() << '}'
         << zakaznici << endl;
    // Pokladnci obsluhujk frontu:
    for(PoklIt i = pokladnici.begin();
      i != pokladnici.end(); i++)
      (*i).run();
    cout << '{' << pokladnici.size() << '}'
         << zakaznici << endl;
    //Jestlie je fronta pli dlouh, pidat dalho pokladnka:
    if(zakaznici.size() / pokladnici.size() > 2)
      pokladnici.push_back(Pokladnik(zakaznici));
    // Jestlie je fronta dostaten krtk, odebrat pokladnka:
    if(pokladnici.size() > 1 &&
      zakaznici.size() / pokladnici.size() < 2)
      for(PoklIt i = pokladnici.begin();
        i != pokladnici.end(); i++)
        if(!(*i).jeZamestnan()) {
          pokladnici.erase(i);
          break; // Konec cyklu for
        }
  }
} ///:~
