//: C07:WildLifeMonitor.cpp
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
#include <algorithm>
#include <cstdlib>
#include <cstddef>
#include <ctime>
#include <iostream>
#include <iterator>
#include <map>
#include <sstream>
#include <string>
#include <vector>
using namespace std;

class DataPoint {
  int x, y; // Souadnice msta
  time_t time; // Doba pozorovn
public:
  DataPoint() : x(0), y(0), time(0) {}
  DataPoint(int xx, int yy, time_t tm) :
    x(xx), y(yy), time(tm) {}

  int getX() const { return x; }
  int getY() const { return y; }
  const time_t* getTime() const { return &time; }
};

string zvire[] = {
  "ipmunk", "jezevec", "svi", "lasika",
  "veverka", "blokur", "medvd", "orel",
  "jestb", "hrabo", "jelen", "vydra", "kolibk",
};
const int ASZ = sizeof zvire/sizeof *zvire;
vector<string> zvirata(zvire, zvire + ASZ);

// vechny inormace jsou obsaeny v 
// objektu "Pozorovani", kter me bt 
// odesln do vstupnho proudu
typedef pair<string, DataPoint> Pozorovani;

ostream&
operator<<(ostream& os, const Pozorovani& s) {
  return os << s.first << " Pozorovn na x= "
    << s.second.getX() << ", y= " << s.second.getY()
    << ", as = " << ctime(s.second.getTime());
}

// Genertor pozorovn:
class GenPozorovani {
  vector<string>& zvirata;
  enum { D = 100 };
public:
  GenPozorovani(vector<string>& an) : zvirata(an) {}
  Pozorovani operator()() {
    Pozorovani vysledek;
    int vyber = rand() % zvirata.size();
    vysledek.first = zvirata[vyber];
    vysledek.second = DataPoint(
      rand() % D, rand() % D, time(0));
    return vysledek;
  }
};

// Zobraz nabdku zvat umoujc uivateli jedno 
// si vybrat, vrac hodnotu indexu:
int menu() {
  cout << "Vyberte si zvire nebo 'k' jako konec: ";
  for(size_t i = 0; i < zvirata.size(); i++)
    cout <<'['<< i <<']'<< zvirata[i] << ' ';
  cout << endl;
  string odpoved;
  cin >> odpoved;
  if(odpoved.at(0) == 'k') return 0;
  istringstream r(odpoved);
  int i;
  r >> i; // pevede na int
  i %= zvirata.size();
  return i;
}

int main() {
  typedef multimap<string, DataPoint> DataMap;
  typedef DataMap::iterator DMIter;
  DataMap pozorovani;
  srand(time(0));  // nhodn sla
  generate_n(inserter(pozorovani, pozorovani.begin()),
    50, GenPozorovani(zvirata));
  // vytisknout ve:
  copy(pozorovani.begin(), pozorovani.end(),
    ostream_iterator<Pozorovani>(cout, ""));
  // Vytisknout pozorovn pro vybran zve:
  for(int count = 1; count < 10; count++) {
    // Pro vbr pouijte menu:
    // int i = menu();
    // generovno nhodn (pro automatick testovn):
    int i = rand() % zvirata.size();
    // itertory v "rozsahu" ukazujc na 
		// zatek a za konec odpovdajcho rozsahu
    pair<DMIter, DMIter> rozsah =
      pozorovani.equal_range(zvirata[i]);
    copy(rozsah.first, rozsah.second,
      ostream_iterator<Pozorovani>(cout, ""));
  }
} ///:~
