//: C10:AbstractFactory.cpp
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// Hrac prosted.
#include <iostream>
using namespace std;

class Prekazka {
public:
  virtual void akce() = 0;
};

class Hrac {
public:
  virtual void bojujeS(Prekazka*) = 0;
};

class Kocicka: public Hrac {
  virtual void bojujeS(Prekazka* pr) {
    cout << "Koika se mus vypodat s pekkou ";
    pr->akce();
  }
};

class KungFuBorec: public Hrac {
  virtual void bojujeS(Prekazka* pr) {
    cout << "Nyn proti sob bojuje KungFuBorec a ";
    pr->akce();
  }
};

class Hlavolam: public Prekazka {
public:
  void akce() { cout << "Hlavolam" << endl; }
};

class HroznaZbran: public Prekazka {
public:
  void akce() { cout << "HroznaZbran" << endl; }
};

// Abstraktn tovrna:
class TovarnaHernichPrvku {
public:
  virtual Hrac* vytvorHrace() = 0;
  virtual Prekazka* vytvorPrekazku() = 0;
};

// Konkrtn tovrny:
class KocickyAHlavolamy : public TovarnaHernichPrvku {
public:
  virtual Hrac* vytvorHrace() { return new Kocicka; }
  virtual Prekazka* vytvorPrekazku() { return new Hlavolam; }
};

class ZabitARozsekat : public TovarnaHernichPrvku {
public:
  virtual Hrac* vytvorHrace() { return new KungFuBorec; }
  virtual Prekazka* vytvorPrekazku() {
    return new HroznaZbran;
  }
};

class HerniProstredi {
  TovarnaHernichPrvku* thp;
  Hrac* h;
  Prekazka* pr;
public:
  HerniProstredi(TovarnaHernichPrvku* tovarna)
  : thp(tovarna), h(tovarna->vytvorHrace()),
    pr(tovarna->vytvorPrekazku()) {}
  void hrat() { h->bojujeS(pr); }
  ~HerniProstredi() {
    delete h;
    delete pr;
    delete thp;
  }
};

int main() {
  HerniProstredi
    g1(new KocickyAHlavolamy),
    g2(new ZabitARozsekat);
  g1.hrat();
  g2.hrat();
}
/* Vstup:
Koika se mus vypodat s pekkou Hlavolam
Nyn proti sob bojuje KungFuBorec a HroznaZbran */ ///:~
