//: C10:MulticastCommand.cpp {RunByHand}
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// Oddlen sprvy udlosti pomoc vzoru Pkaz
#include <iostream>
#include <vector>
#include <string>
#include <ctime>
#include <cstdlib>
using namespace std;

// Rmec pro spoutn kol:
class Ukol {
public:
  virtual void operace() = 0;
};

class SpoustecUkolu {
  static vector<Ukol*> ukoly;
  SpoustecUkolu() {} // Vytvoit z nho Jedinka
  SpoustecUkolu& operator=(SpoustecUkolu&); // Zakzno
  SpoustecUkolu(const SpoustecUkolu&); // Zakzno
  static SpoustecUkolu su;
public:
  static void pridat(Ukol& t) { ukoly.push_back(&t); }
  static void spustit() {
    vector<Ukol*>::iterator it = ukoly.begin();
    while(it != ukoly.end())
      (*it++)->operace();
  }
};

SpoustecUkolu SpoustecUkolu::su;
vector<Ukol*> SpoustecUkolu::ukoly;

class SimulatorUdalosti {
  clock_t vytvoreni;
  clock_t zpozdeni;
public:
  SimulatorUdalosti() : vytvoreni(clock()) {
    zpozdeni = CLOCKS_PER_SEC/4 * (rand() % 20 + 1);
    cout << "zpozdeni = " << zpozdeni << endl;
  }
  bool vyvolana() {
    return clock() > vytvoreni + zpozdeni;
  }
};

// Tda generujc asynchronn udlosti:
class Tlacitko {
  bool stisknuto;
  string id;
  SimulatorUdalosti e; // na ukzku
public:
  Tlacitko(string nazev) : stisknuto(false), id(nazev) {}
  void stisknout() { stisknuto = true; }
  bool jeStisknuto() {
    if(e.vyvolana()) stisknout(); // Simuluje udlost
    return stisknuto;
  }
  friend ostream&
  operator<<(ostream& os, const Tlacitko& b) {
    return os << b.id;
  }
};

// objekt Pkazu
class OverTlacitko : public Ukol {
  Tlacitko& tlacitko;
  bool zpracovano;
public:
  OverTlacitko(Tlacitko & b) : tlacitko(b), zpracovano(false) {}
  void operace() {
    if(tlacitko.jeStisknuto() && !zpracovano) {
      cout << tlacitko << " stisknuto" << endl;
      zpracovano = true;
    }
  }
};

// Procedury provdjc hlavn innost. Je poteba, 
// obas je "peruit" aby zkontrolovaly stav tlatek 
// a ostan udlosti:
void procedura1() {
  // Zde se provd operace funkce procedura1
  // ...
  SpoustecUkolu::spustit(); // Zkontrolovat vechny udlosti
}

void procedura2() {
  // Zde se provd operace funkce procedura2
  // ...
  SpoustecUkolu::spustit(); // Zkontrolovat vechny udlosti
}

void procedura3() {
  // Zde se provd operace funkce procedura3
  // ...
  SpoustecUkolu::spustit(); // Zkontrolovat vechny udlosti
}

int main() {
  srand(time(0)); // genertor nhodnch sel
  Tlacitko b1("Tlatko 1"), b2("Tlatko 2"), b3("Tlatko 3");
  OverTlacitko cb1(b1), cb2(b2), cb3(b3);
  SpoustecUkolu::pridat(cb1);
  SpoustecUkolu::pridat(cb2);
  SpoustecUkolu::pridat(cb3);
  cout << "Ukonit stiskem Control-C" << endl;
  while(true) {
    procedura1();
    procedura2();
    procedura3();
  }
} ///:~
