//: C11:CarBuilder.cpp {RunByHand}
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (a) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// Jak funguje funkce broadcast().
//{L} ZThread
#include <iostream>
#include <string>
#include "zthread/Thread.h"
#include "zthread/Mutex.h"
#include "zthread/Guard.h"
#include "zthread/Condition.h"
#include "zthread/ThreadedExecutor.h"
#include "TQueue.h"
using namespace ZThread;
using namespace std;

class Auto {
  int id;
  bool motor, prevodovka, kola;
public:
  Auto(int idn) : id(idn), motor(false),
  prevodovka(false), kola(false) {}
  // Przdn objekt Auto:
  Auto() : id(-1), motor(false),
  prevodovka(false), kola(false) {}
  // Nesybnchronizovno -- Pedpokldaj se 
	// atomick operace s typem bool:
  int getId() { return id; }
  void pridatMotor() { motor = true; }
  bool motorNamontovan() { return motor; }
  void pridatPrevodovku() { prevodovka = true; }
  bool prevodovkaNamontovana() { return prevodovka; }
  void pridatKola() { kola = true; }
  bool kolaNamontovana() { return kola; }
  friend ostream& operator<<(ostream& os, const Auto& a) {
    return os << "Auto " << a.id << " ["
      << " motor: " << a.motor
      << " prevodovka: " << a.prevodovka
      << " kola: " << a.kola << " ]";
  }
};

typedef CountedPtr< TQueue<Auto> > FrontaAut;

class StavitelRamu : public Runnable {
  FrontaAut frontaAut;
  int pocitadlo;
public:
  StavitelRamu(FrontaAut& fa) : frontaAut(fa),pocitadlo(0) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        Thread::sleep(1000);
        // Vytvoit rm:
        Auto a(pocitadlo++);
        cout << a << endl;
        // vloit do fronty
        frontaAut->put(a);
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "StavitelRamu ukonen" << endl;
  }
};

class Drzak {
  Auto a; // Uchovv auto, na kterm se prv pracuje
  bool obsazen;
  Mutex zamekPrace, zamekHotovo;
  Condition podminkaPrace, podminkaHotovo;
  bool pracujeMotorovyRobot, pracujeKolovyRobot, 
			 pracujePrevodovkovyRobot;
public:
  Drzak()
  : podminkaPrace(zamekPrace), podminkaHotovo(zamekHotovo) {
    obsazen = false;
    pracujeMotorovyRobot = true;
    pracujeKolovyRobot = true;
    pracujePrevodovkovyRobot = true;
  }
  void vlozitAuto(Auto ram) {
    a = ram;
    obsazen = true;
  }
  Auto sundatAuto() { // Me vythnout auto jen jednou
    if(!obsazen) {
      cerr << "v Drzaku neni zadne auto, kter by lo sundat pomoc sundatAuto()" << endl;
      return Auto(); // "Nulov" objekt Auto
    }
    obsazen = false;
    return a;
  }
  // pstup k autu v drku:
  Auto* operator->() { return &a; }
	// Umonit robotm nabdnout sv sluby drku:
  void nabidnoutSluzbyMotorBota() {
    Guard<Mutex> g(zamekPrace);
    while(pracujeMotorovyRobot)
      podminkaPrace.wait();
    pracujeMotorovyRobot = true; // Pijmout jeho prci
  }
  void nabidnoutSluzbyKolBota() {
    Guard<Mutex> g(zamekPrace);
    while(pracujeKolovyRobot)
      podminkaPrace.wait();
    pracujeKolovyRobot = true; // Pijmout jeho prci
  }
  void nabidnoutSluzbyPrevodBota() {
    Guard<Mutex> g(zamekPrace);
    while(pracujePrevodovkovyRobot)
      podminkaPrace.wait();
    pracujePrevodovkovyRobot = true; // Pijmout jeho prci
  }
  // ct ekajcm robotm, e je prce pipravena
  void zacitPracovat() {
    Guard<Mutex> g(zamekPrace);
    pracujeMotorovyRobot = false;
    pracujeKolovyRobot = false;
    pracujePrevodovkovyRobot = false;
    podminkaPrace.broadcast();
  }
  // Kad robot nahls, e je hotov
  void ukolDokoncen() {
    Guard<Mutex> g(zamekHotovo);
    podminkaHotovo.signal();
  }
  // Vedouci ek, a budou splnny vechny koly.
  void cekejNaSkonceniPrace() {
    Guard<Mutex> g(zamekHotovo);
    while(!(a.motorNamontovan() && a.prevodovkaNamontovana()
            && a.kolaNamontovana()))
      podminkaHotovo.wait();
  }
};

typedef CountedPtr<Drzak> DrzakPtr;

class Vedouci : public Runnable {
  FrontaAut frontaRamu, frontaHotovych;
  DrzakPtr drzak;
public:
  Vedouci(FrontaAut& fa, FrontaAut& fq, DrzakPtr dr)
  : frontaRamu(fa), frontaHotovych(fq), drzak(dr) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Blokovat, dokud nen k dispozici rm:
        drzak->vlozitAuto(frontaRamu->get());
        // Upozornit roboty, e auto je pipraveno:
        drzak->zacitPracovat();
        // ekar, ne je prce dokonena:
        drzak->cekejNaSkonceniPrace();
        // Vloit auto do fronty pro dal koly:
        frontaHotovych->put(drzak->sundatAuto());
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "Vedouci ukonen" << endl;
  }
};

class MotorovyRobot : public Runnable {
  DrzakPtr drzak;
public:
  MotorovyRobot(DrzakPtr dr) : drzak(dr) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Blokovat dokud nen nabdnuta/pijata prce:
        drzak->nabidnoutSluzbyMotorBota();
        cout << "Instaluji motor" << endl;
        (*drzak)->pridatMotor();
        drzak->ukolDokoncen();
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "MotorovyRobot ukonen" << endl;
  }
};

class PrevodovkovyRobot : public Runnable {
  DrzakPtr drzak;
public:
  PrevodovkovyRobot(DrzakPtr dr) : drzak(dr) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Blokovat dokud nen nabdnuta/pijata prce:
        drzak->nabidnoutSluzbyPrevodBota();
        cout << "Instaluji pevodovku" << endl;
        (*drzak)->pridatPrevodovku();
        drzak->ukolDokoncen();
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "PrevodovkovyRobot ukonen" << endl;
  }
};

class KolovyRobot : public Runnable {
  DrzakPtr drzak;
public:
  KolovyRobot(DrzakPtr dr) : drzak(dr) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Blokovat dokud nen nabdnuta/pijata prce:
        drzak->nabidnoutSluzbyKolBota();
        cout << "Instaluji Kola" << endl;
        (*drzak)->pridatKola();
        drzak->ukolDokoncen();
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "KolovyRobot ukonen" << endl;
  }
};

class Reporter : public Runnable {
  FrontaAut frontaAut;
public:
  Reporter(FrontaAut& fa) : frontaAut(fa) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        cout << frontaAut->get() << endl;
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "Reporter ukonen" << endl;
  }
};

int main() {
  cout << "Ukonit stiskem klvesy <Enter>" << endl;
  try {
    FrontaAut frontaRamu(new TQueue<Auto>),
             frontaHotovych(new TQueue<Auto>);
    DrzakPtr drzak(new Drzak);
    ThreadedExecutor montazniLinka;
    montazniLinka.execute(new MotorovyRobot(drzak));
    montazniLinka.execute(new PrevodovkovyRobot(drzak));
    montazniLinka.execute(new KolovyRobot(drzak));
    montazniLinka.execute(
      new Vedouci(frontaRamu, frontaHotovych, drzak));
    montazniLinka.execute(new Reporter(frontaHotovych));
    // Vechno spustit zahjnm vroby rm:
    montazniLinka.execute(new StavitelRamu(frontaRamu));
    cin.get();
    montazniLinka.interrupt();
  } catch(Synchronization_Exception& e) {
    cerr << e.what() << endl;
  }
} ///:~
