//: C11:DiningPhilosophers.h
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// Tdy pro veec filozofy.
#ifndef DININGPHILOSOPHERS_H
#define DININGPHILOSOPHERS_H
#include <string>
#include <iostream>
#include <cstdlib>
#include "zthread/Condition.h"
#include "zthread/Guard.h"
#include "zthread/Mutex.h"
#include "zthread/Thread.h"
#include "Display.h"

class Tycinka {
  ZThread::Mutex zamek;
  ZThread::Condition polozena;
  bool zvednuta;
public:
  Tycinka() : polozena(zamek), zvednuta(false) {}
  void vzit() {
    ZThread::Guard<ZThread::Mutex> g(zamek);
    while(zvednuta)
      polozena.wait();
    zvednuta = true;
  }
  void polozit() {
    ZThread::Guard<ZThread::Mutex> g(zamek);
    zvednuta = false;
    polozena.signal();
  }
};

class Filozof : public ZThread::Runnable {
  Tycinka& leva;
  Tycinka& prava;
  int id;
  int faktorPremysleni;
  ZThread::CountedPtr<Display> display;
  int nahodnaDobaSpanku() {
    if(faktorPremysleni == 0) return 0;
    return rand()/(RAND_MAX/faktorPremysleni) * 250;
  }
  void output(std::string s) {
    std::ostringstream os;
    os << *this << " " << s << std::endl;
    display->output(os);
  }
public:
  Filozof(Tycinka& l, Tycinka& r,
  ZThread::CountedPtr<Display>& disp, int ident, int premyslet)
  : leva(l), prava(r), id(ident), faktorPremysleni(premyslet),
    display(disp) {}
  virtual void run() {
    try {
      while(!ZThread::Thread::interrupted()) {
        output("Pemlen");
        ZThread::Thread::sleep(nahodnaDobaSpanku());
        // hlavovjc
        output("Beru pravou");
        prava.vzit();
        output("Beru levou");
        leva.vzit();
        output("Jm");
        ZThread::Thread::sleep(nahodnaDobaSpanku());
        prava.polozit();
        leva.polozit();
      }
    } catch(ZThread::Synchronization_Exception& e) {
      output(e.what());
    }
  }
  friend std::ostream&
  operator<<(std::ostream& os, const Filozof& f) {
    return os << "Filozof " << f.id;
  }
};
#endif // DININGPHILOSOPHERS_H ///:~
