//: C11:ToastOMaticMarkII.cpp {RunByHand}
// Z knihy "Myslme v jazyce C++, st 2" od Bruce Eckela a Chucka Allisona.
// (c) 1995-2004 MindView, Inc. Vechna prva vyhrazena.
// Pette si podmnky pouvn zdrojovch kd umstn v souboru 
// 'Licence.txt' dodvanm spolu s balkem kdu dostupnm na www.MindView.net.
// een problm pomoc TQueue
//{L} ZThread
#include <iostream>
#include <string>
#include <cstdlib>
#include <ctime>
#include "zthread/Thread.h"
#include "zthread/Mutex.h"
#include "zthread/Guard.h"
#include "zthread/Condition.h"
#include "zthread/ThreadedExecutor.h"
#include "TQueue.h"
using namespace ZThread;
using namespace std;

class Topinka {
  enum Stav { SUCHA, S_MASLEM, S_MARMELADOU };
  Stav stav;
  int id;
public:
  Topinka(int idn) : stav(SUCHA), id(idn) {}
  #ifdef __DMC__ // nesprvn vyaduje vchoz
  Topinka() { assert(0); } // Nemlo by se nikdy zavolat
  #endif
  void maslo() { stav = S_MASLEM; }
  void marmelada() { stav = S_MARMELADOU; }
  string zjistiStav() const {
    switch(stav) {
      case SUCHA: return "such";
      case S_MASLEM: return "s mslem";
      case S_MARMELADOU: return "s marmeldou";
      default: return "Chyba";
    }
  }
  int getId() { return id; }
  friend ostream& operator<<(ostream& os, const Topinka& t) {
    return os << "Topinka " << t.id << ": " << t.zjistiStav();
  }
};

typedef CountedPtr< TQueue<Topinka> > FrontaTopinek;

class Topinkovac : public Runnable {
  FrontaTopinek frontaTopinek;
  int pocet;
public:
  Topinkovac(FrontaTopinek& tq) : frontaTopinek(tq), pocet(0) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        int zpozdeni = rand()/(RAND_MAX/5)*100;
        Thread::sleep(zpozdeni);
        // upct topinku
        Topinka t(pocet++);
        cout << t << endl;
        // Pidat do fronty
        frontaTopinek->put(t);
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "Nedopeen topinka" << endl;
  }
};

// Namae mslo na topinku:
class Maslo : public Runnable {
  FrontaTopinek frontaSuchych, frontaNamazanych;
public:
  Maslo(FrontaTopinek& sucha, FrontaTopinek& sMaslem)
  : frontaSuchych(sucha), frontaNamazanych(sMaslem) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Zablokovno dokud nen k dispozici dal topinka
        Topinka t = frontaSuchych->get();
        t.maslo();
        cout << t << endl;
        frontaNamazanych->put(t);
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "Bez msla" << endl;
  }
};

// Namazat marmeldu na topinku s mslem:
class Marmelada : public Runnable {
  FrontaTopinek frontaNamazanych, frontaHotovych;
public:
  Marmelada(FrontaTopinek& sMaslem, FrontaTopinek& hotova)
  : frontaNamazanych(sMaslem), frontaHotovych(hotova) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Zablokovno dokud nen k dispozici dal topinka
        Topinka t = frontaNamazanych->get();
        t.marmelada();
        cout << t << endl;
        frontaHotovych->put(t);
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "bez Marmeldy" << endl;
  }
};

// J topinky:
class Jedlik : public Runnable {
  FrontaTopinek frontaHotovych;
  int pocitadlo;
public:
  Jedlik(FrontaTopinek& hotova)
  : frontaHotovych(hotova), pocitadlo(0) {}
  void run() {
    try {
      while(!Thread::interrupted()) {
        // Zablokovno dokud nen k dispozici dal topinka:
        Topinka t = frontaHotovych->get();
        // Ovuje, e topinky jdou ve sprvnm poad
        // a e jsou vechny s marmeldou:
        if(t.getId() != pocitadlo++ ||
           t.zjistiStav() != "s marmeldou") {
          cout << ">>>> Chyba: " << t << endl;
          exit(1);
        } else
          cout << "Chramst! " << t << endl;
      }
    } catch(Interrupted_Exception&) { /* Konec */ }
    cout << "Dojedeno" << endl;
  }
};

int main() {
  srand(time(0)); // Inicializovat genertor nhodnch sel
  try {
    FrontaTopinek frontaSuchych(new TQueue<Topinka>),
               frontaNamazanych(new TQueue<Topinka>),
               frontaHotovych(new TQueue<Topinka>);
    cout << "Konec stiskem klvesy <Enter>" << endl;
    ThreadedExecutor executor;
    executor.execute(new Topinkovac(frontaSuchych));
    executor.execute(new Maslo(frontaSuchych,frontaNamazanych));
    executor.execute(
      new Marmelada(frontaNamazanych, frontaHotovych));
    executor.execute(new Jedlik(frontaHotovych));
    cin.get();
    executor.interrupt();
  } catch(Synchronization_Exception& e) {
    cerr << e.what() << endl;
  }
} ///:~
